@@ -40,14 +40,26 @@ int
 find_bus_ancestry( struct sysfs_device* dev, char** buses ) {
     char **i;
 
-    if( !dev || !buses )
+    if( !buses ) {
+        debug ( "find_bus_ancestry: no buses to check, fail\n" );
         return 0;
-
-    for( i = buses; *i; ++i ) {
-        if( !strcmp( dev->bus, *i ) )
-            return 1;
     }
 
+    if( !dev ) {
+        debug ( "find_bus_ancestry: dev == NULL, fail\n" );
+        return 0;
+    }
+
+    for( i = buses; *i; ++i ) {
+        if( !strcmp( dev->bus, *i ) ) {
+            debug ( "find_bus_ancestry: device %s (path %s, bus %s) matches query, success\n", 
+                    dev->name, dev->path, dev->bus );
+            return 1;
+        }
+    }
+
+    debug ( "find_bus_ancestry: device %s (path %s, bus %s) does not match, trying parent\n", 
+            dev->name, dev->path, dev->bus );
     return find_bus_ancestry( sysfs_get_device_parent( dev ), buses );
 }
 
@@ -122,7 +134,7 @@ find_sysfs_device( const char* dev, char* blockdevpath, size_t blockdevpathsize
             if( sysminor != devminor ) {
                 int found_part = 0;
                 
-                debug( "find_sysfs_device: minor device numbers do not match, checking partitions...");
+                debug( "find_sysfs_device: minor device numbers do not match, checking partitions...\n");
 
                 partdir = opendir( devdirname );
                 if( !partdir ) {
@@ -389,13 +401,17 @@ device_whitelisted( const char* device )
     char *d;
     regex_t re;
     regmatch_t match[3];
+    int result;
+    const char* whitelist_regex = "^[[:space:]]*([a-zA-Z0-9/_+.-]+)[[:space:]]*(#.*)?$";
 
     fwl = fopen( WHITELIST, "r" );
     if( !fwl )
         return 0;
 
-    if( regcomp( &re, "^[[:space:]]*([a-zA-Z0-9/_+\\-\\.]+)[[:space:]]*(#.*)?$", REG_EXTENDED ) ) {
-        fprintf( stderr, "Internal error: device_whitelisted(): could not compile regex\n" );
+    result = regcomp( &re, whitelist_regex, REG_EXTENDED );
+    if( result ) {
+        regerror( result, &re, line, sizeof( line ) );
+        fprintf( stderr, "Internal error: device_whitelisted(): could not compile regex: %s\n", line );
         exit( -1 );
     }
 
