@@ -17,6 +17,7 @@
 #include <limits.h>
 #include <getopt.h>
 #include <libintl.h>
+#include <locale.h>
 
 #include "policy.h"
 #include "utils.h"
@@ -118,10 +119,10 @@ do_umount( const char* device, int do_lazy )
     int status;
 
     if( do_lazy )
-        status = spawn( SPAWN_EROOT|SPAWN_RROOT, UMOUNTPROG, UMOUNTPROG, "-l",
+        status = spawnl( SPAWN_EROOT|SPAWN_RROOT, UMOUNTPROG, UMOUNTPROG, "-l",
                 device, NULL );
     else
-        status = spawn( SPAWN_EROOT|SPAWN_RROOT, UMOUNTPROG, UMOUNTPROG,
+        status = spawnl( SPAWN_EROOT|SPAWN_RROOT, UMOUNTPROG, UMOUNTPROG,
                 device, NULL );
 
     if( status != 0 ) {
